@@ -27,12 +27,18 @@
 #include "features.hpp"
 #include "config.hpp"
 #include "autosaver.hpp"
+#include "autosave_folder.hpp"
 #include "command_executer.hpp"
 
 #ifdef WITH_ZEROCONF
 # include <obby/zeroconf.hpp>
 #endif
 
+#ifdef WITH_AVAHI
+# include <obby/zeroconf_avahi.hpp>
+# include <avahi-glib/glib-watch.h>
+#endif
+
 namespace Sobby
 {
 
@@ -56,6 +62,8 @@ public:
 	bool on_cmd_help(const ArgList& args);
 	bool on_cmd_users(const ArgList& args);
 	bool on_cmd_documents(const ArgList& args);
+
+	const std::string& get_command_dir() const;
 protected:
 	virtual bool on_stdin(Glib::IOCondition condition);
 
@@ -63,13 +71,18 @@ protected:
 
 	int m_port;
 	bool m_interactive;
+	std::string m_command_dir;
 
 	Glib::RefPtr<Glib::MainLoop> m_main_loop;
 	std::auto_ptr<ServerBuffer> m_server;
 	std::auto_ptr<AutoSaver> m_autosaver;
+	std::auto_ptr<AutoSaveFolder> m_autosave_folder;
 	std::auto_ptr<CommandExecuter> m_command_executer;
 #ifdef WITH_ZEROCONF
-	std::auto_ptr<obby::zeroconf> m_zeroconf;
+	std::auto_ptr<obby::zeroconf_base> m_zeroconf;
+#endif
+#ifdef WITH_AVAHI
+	AvahiGLibPoll* m_glib_poll;
 #endif
 
 	static const CommandMap& m_cmd_map;
