@@ -17,6 +17,7 @@
  */
 
 #include <iostream>
+#include <glibmm/fileutils.h>
 #include <glibmm/optionentry.h>
 #include <glibmm/optiongroup.h>
 #include <glibmm/optioncontext.h>
@@ -58,12 +59,16 @@ const Sobby::Server::CommandMap& Sobby::Server::m_cmd_map = create_cmd_map();
 Sobby::Server::Server(int argc, char* argv[]):
 	m_interactive(false),
 	m_main_loop(Glib::MainLoop::create() )
+#ifdef WITH_AVAHI
+	,m_glib_poll(NULL)
+#endif
 {
 	Glib::ustring name;
 	Glib::ustring password;
 	const char* session = NULL;
 
 	std::string autosave_file;
+	std::string autosave_folder;
 	int autosave_interval;
 
 	std::string config_file_read;
@@ -112,21 +117,33 @@ Sobby::Server::Server(int argc, char* argv[]):
 		//m_interactive = config->get_value<bool>(
 		//	"interactive", false
 		//);
+
+		autosave_folder = entry.get_value<std::string>(
+			"autosave_directory", ""
+		);
+
 		autosave_file = entry.get_value<std::string>(
 			"autosave_file", "autosave.obby"
 		);
+
 		autosave_interval = entry.get_value<unsigned int>(
 			"autosave_interval", 0
 		);
 		password = entry.get_value<Glib::ustring>("password", "");
+
+		m_command_dir = entry.get_value<Glib::ustring>(
+			"command_directory", ""
+		);
 	}
 	else
 	{
 		m_port = 6522;
 		name = "Standalone obby server";
+		autosave_folder = "";
 		autosave_file = "autosave.obby";
 		autosave_interval = 0;
 		password = "";
+		m_command_dir = "";
 	}
 
 	// Parse another time to get remaining options
@@ -151,11 +168,19 @@ Sobby::Server::Server(int argc, char* argv[]):
 		"Show prompt to enter commands at run-time"
 	);
 
+	Glib::OptionEntry opt_common_autosave_folder;
+	opt_common_autosave_folder.set_long_name("autosave-directory");
+	opt_common_autosave_folder.set_arg_description("FOLDER");
+	opt_common_autosave_folder.set_description(
+		"Folder where to store autosaved documents. "
+	);
+
 	Glib::OptionEntry opt_common_autosave_file;
 	opt_common_autosave_file.set_long_name("autosave-file");
 	opt_common_autosave_file.set_arg_description("FILE");
 	opt_common_autosave_file.set_description(
-		"File where to store autosaved sessions"
+		"File where to store autosaved sessions. "
+		"Ignored if autosave-directory is set."
 	);
 
 	Glib::OptionEntry opt_common_autosave_interval;
@@ -165,6 +190,14 @@ Sobby::Server::Server(int argc, char* argv[]):
 		"Interval (in seconds) between autosaves; 0 disables autosave"
 	);
 
+	Glib::OptionEntry opt_common_command_dir;
+	opt_common_command_dir.set_long_name("command-directory");
+	opt_common_command_dir.set_arg_description("DIRECTORY");
+	opt_common_command_dir.set_description(
+		"Directory where to find executable scripts which clients "
+		"might execute"
+	);
+
 	Glib::OptionEntry opt_net_port;
 	opt_net_port.set_short_name('p');
 	opt_net_port.set_long_name("port");
@@ -190,6 +223,11 @@ Sobby::Server::Server(int argc, char* argv[]):
 	opt_group_common.add_entry(opt_common_name, name);
 	opt_group_common.add_entry(opt_common_interactive, m_interactive);
 
+	opt_group_common.add_entry_filename(
+		opt_common_autosave_folder,
+		autosave_folder
+	);
+
 	opt_group_common.add_entry_filename(
 		opt_common_autosave_file,
 		autosave_file
@@ -200,6 +238,11 @@ Sobby::Server::Server(int argc, char* argv[]):
 		autosave_interval
 	);
 
+	opt_group_common.add_entry_filename(
+		opt_common_command_dir,
+		m_command_dir
+	);
+
 	// To display config file in help output
 	opt_group_common.add_entry_filename(
 		opt_config_file_read,
@@ -215,7 +258,7 @@ Sobby::Server::Server(int argc, char* argv[]):
 
 	opt_group_auth.add_entry(opt_auth_password, password);
 
-	Glib::OptionContext opt_ctx("[session]");
+	Glib::OptionContext opt_ctx("[file1 or session] [file2] [...]");
 	opt_ctx.set_help_enabled(true);
 
 	opt_ctx.set_main_group(opt_group_common);
@@ -244,6 +287,11 @@ Sobby::Server::Server(int argc, char* argv[]):
 				"name", "Standalone obby server"
 			);
 
+		if(autosave_folder.empty())
+			autosave_folder = entry.get_value<std::string>(
+				"autosave_directory", ""
+			);
+
 		if(autosave_file.empty() )
 			autosave_file = entry.get_value<std::string>(
 				"autosave_file", "autosave.obby"
@@ -258,14 +306,21 @@ Sobby::Server::Server(int argc, char* argv[]):
 			password = entry.get_value<Glib::ustring>(
 				"password", ""
 			);
+
+		if(m_command_dir.empty())
+			m_command_dir = entry.get_value<Glib::ustring>(
+				"command_directory", ""
+			);
 	}
 	else
 	{
 		if(m_port == 0) m_port = 6522;
 		if(name.empty() ) name = "Standalone obby server";
+		if(autosave_folder.empty()) autosave_folder = "";
 		if(autosave_file.empty() ) autosave_file = "autosave.obby";
 		if(autosave_interval == 0) autosave_interval = 0;
 		if(password.empty() ) password = "";
+		if(m_command_dir.empty()) m_command_dir = "";
 	}
 
 	if(!config_file_write.empty() )
@@ -280,8 +335,10 @@ Sobby::Server::Server(int argc, char* argv[]):
 		entry.set_value("port", m_port);
 		entry.set_value("name", name);
 		entry.set_value("autosave_file", autosave_file);
+		entry.set_value("autosave_directory", autosave_folder);
 		entry.set_value("autosave_interval", autosave_interval);
 		entry.set_value("password", password);
+		entry.set_value("command_directory", m_command_dir);
 
 		config.save(config_file_write);
 
@@ -294,40 +351,106 @@ Sobby::Server::Server(int argc, char* argv[]):
 
 	// Start server
 	m_server.reset(new ServerBuffer);
+	m_server->set_enable_keepalives(true);
 
 	if(session == NULL)
+	{
 		m_server->open(m_port);
+	}
 	else
-		m_server->open(session, m_port);
+	{
+		try
+		{
+			m_server->open(session, m_port);
+		}
+		catch(std::exception& e)
+		{
+			// It is not an obby session file, so load it as
+			// normal document.
+			if(m_server->is_open()) m_server->close();
+			m_server->open(m_port);
+
+			for(int i = 1; i < argc; ++ i)
+			{
+				std::string content = Glib::file_get_contents(
+					argv[i]
+				);
+
+				if(!g_utf8_validate(content.c_str(), content.length(), NULL))
+				{
+					obby::format_string errstring("The file '%0%' does not contain valid UTF-8. Please convert it to UTF-8 before loading it into sobby.");
+					errstring << argv[i];
+					throw std::runtime_error(errstring.str());
+				}
+
+				m_server->document_create(argv[i], "UTF-8", content);
+			}
+		}
+	}
 
 	m_server->set_global_password(password);
 
 	if(autosave_interval > 0)
 	{
-		m_autosaver.reset(
-			new AutoSaver(
-				*m_server,
-				autosave_file,
-				autosave_interval
-			)
-		);
+		if(autosave_folder.empty())
+		{
+			m_autosaver.reset(
+				new AutoSaver(
+					*m_server,
+					autosave_file,
+					autosave_interval
+				)
+			);
 
-		m_autosaver->error_event().connect(
-			sigc::mem_fun(*this, &Server::on_autosave_error) );
+			m_autosaver->error_event().connect(
+				sigc::mem_fun(*this, &Server::on_autosave_error) );
+		}
+		else
+		{
+			if(!Glib::file_test(autosave_folder, Glib::FILE_TEST_IS_DIR))
+			{
+				obby::format_string str("'%0%' is not a directory");
+				str << autosave_folder;
+				throw std::runtime_error(str.str()); 
+			}
+
+			m_autosave_folder.reset(
+				new AutoSaveFolder(
+					*m_server,
+					autosave_folder,
+					autosave_interval
+				)
+			);
+
+			m_autosave_folder->error_event().connect(
+				sigc::mem_fun(*this, &Server::on_autosave_error) );
+		}
 	}
 
-	m_command_executer.reset(new CommandExecuter(*m_server) );
+	m_command_executer.reset(new CommandExecuter(*this, *m_server) );
 
 #ifdef WITH_ZEROCONF
 	try
 	{
+#ifdef WITH_AVAHI
+		m_glib_poll = avahi_glib_poll_new(NULL, G_PRIORITY_DEFAULT);
+		m_zeroconf.reset(new obby::zeroconf_avahi(avahi_glib_poll_get(m_glib_poll)));
+#else
 		m_zeroconf.reset(new obby::zeroconf);
+		// Process zeroconf events periodically
+		Glib::signal_timeout().connect(sigc::bind(sigc::mem_fun(*m_zeroconf.get(), &obby::zeroconf_base::select), 0), 1500);
+#endif
 		m_zeroconf->publish(name, m_port);
 	}
 	catch(std::runtime_error&)
 	{
+#ifdef WITH_AVAHI
+		std::cerr << "ERROR: Avahi initialisation failed. Please run "
+		          << "avahi-daemon prior to Sobby." << std::endl;
+#else
 		std::cerr << "ERROR: Howl initialisation failed. Please run "
 		          << "mDNSResponder prior to Sobby." << std::endl;
+#endif
 		std::cerr << "       Zeroconf support is thus deactivated for "
 		          << "this session." << std::endl;
 		m_zeroconf.reset();
@@ -337,6 +460,12 @@ Sobby::Server::Server(int argc, char* argv[]):
 
 Sobby::Server::~Server()
 {
+#ifdef WITH_AVAHI
+	if(m_glib_poll != NULL)
+	{
+		avahi_glib_poll_free(m_glib_poll);
+	}
+#endif
 }
 
 void Sobby::Server::run()
@@ -474,3 +603,8 @@ bool Sobby::Server::on_cmd_documents(const ArgList& args)
 	std::cout << m_server->document_count() << " documents" << std::endl;
 	return true;
 }
+
+const std::string& Sobby::Server::get_command_dir() const
+{
+	return m_command_dir;
+}
